package com.nisum.paymentvendorsimulator.creditcard.data;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.nisum.paymentvendorsimulator.creditcard.domain.dto.CreditCardResponse;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;

@Component
public class DataServiceClient {
    @Value("${data.endpoint}")
    private String dataServiceEndpoint;
    private final ObjectMapper objectMapper = new ObjectMapper();

    private CreditCardResponse convertResponse(String rawResponse) throws JsonProcessingException {
        CreditCardResponse res = objectMapper.readValue(rawResponse.toString(), CreditCardResponse.class);
        return res;
    }

    public void saveAuthorization(CreditCardResponse ccData) throws IOException {
        URL url = new URL(dataServiceEndpoint);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setRequestMethod("POST");
        connection.setRequestProperty("Content-Type", "application/json");
        connection.setDoOutput(true);
        // Attach the body
        String jsonBody = objectMapper.writeValueAsString(ccData);
        try(OutputStream os = connection.getOutputStream()) {
            byte[] input = jsonBody.getBytes(StandardCharsets.UTF_8);
            os.write(input, 0, input.length);
        }
        // Read the response
        try(BufferedReader br = new BufferedReader(
            new InputStreamReader(connection.getInputStream(), StandardCharsets.UTF_8))) {
            StringBuilder response = new StringBuilder();
            String responseLine = null;
            while ((responseLine = br.readLine()) != null) {
                response.append(responseLine.trim());
            }
            System.out.println(response.toString());

        }
        connection.disconnect();
    }

    public CreditCardResponse getAuthorization(String id) throws IOException {
        URL url = new URL(dataServiceEndpoint + "/" + id);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setRequestMethod("GET");
        connection.setRequestProperty("Content-Type", "application/json");
        int status = connection.getResponseCode();
        if (status >= 200 & status < 300) {
            System.out.println("SUCCESS");
            BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
            String inputLine;
            StringBuffer content = new StringBuffer();
            while ((inputLine = in.readLine()) != null) {
                content.append(inputLine);
            }
            in.close();
            System.out.println(content.toString());
            CreditCardResponse res = convertResponse(content.toString());
            return res;
        } else {
            System.out.println("ERROR");
        }
        connection.disconnect();
        return null;
    }
}
